﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOWDMNet;  // the namespace exposes the AIOWDM Class interface 

namespace DIO
{
    public partial class Form1 : Form
    {
        // One and only Device:
        public Int32 CardNum = 0;   // assume the first count connected to AIOWDM is the correct type for this sample
        public UInt32 Channels = 8; 

        bool RunFlag = true;

        public UInt32[] value = new UInt32[] { 0, 0, 0 };  // 3 values used for read write
        public int i = 0; // just for demo purposes its incremented every loop never reset or wraps

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();


        public Form1()
        {
            InitializeComponent();

            // Initialize default values:

            // Ping the driver AIOWDM.SYS:
            // This is the one place we actual might use all 16 bits returned for an error code AA55 
            // otherwise we only use an 8 bit byte of data per read and write:
            if (AIOWDM.InPortB(0x61) == 0xAA55) //Driver Call  this is an absolute address no relative address function
            {
                MessageBox.Show("The DLL can't find any attached SYS file (AIOWDM.dll -> AIOWDM.sys)\nEnsure that a board is installed properly: check Device Manager for problems or missing cards.", "Warning");
            }

            // Add the event and the event handler for the method that will 
            // process the timer event to the timer:
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            // Set the timer interval in miliseconds and start this has a 55ms min resolution:
            myTimer.Interval = 1000;  // sec
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            CardNum = 0; // "the first card found"
            // software board reset: 
            AIOWDM.RelOutPortDWord(CardNum, 0x0f, 0);
            // disable all COS IRQs: 
            AIOWDM.RelOutPortB(CardNum, 0x0e, 0);
            // disable all filters, enable buffer (disable tri-state): 
            AIOWDM.RelOutPortB(CardNum, 0x0c, 0x00 | 1);
            // clear any pending CoS IRQ flags: 
            AIOWDM.RelOutPortDWord(CardNum, 0x08, 0xFFFFFFFF); 

        }

        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            myTimer.Stop();
            UpdateGUIState();
            myTimer.Enabled = true;
        }

        private string pad32(string s)
        {
            int Len, bit = 0;

            Len = s.Length;

            for (bit = Len; bit < 32; bit++)
                s = "0" + s;
            return s;
        }

        private void UpdateGUIState()
        {
            UInt32 readbacks, inputs, coss;
            
            // readback outputs
            readbacks = AIOWDM.RelInPortDWord(CardNum, 0);
            lblReadback.Text = pad32(Convert.ToString(readbacks, 2));
            
            // read inputs
            inputs = AIOWDM.RelInPortDWord(CardNum, 4);
            lblInputs.Text = pad32(Convert.ToString(inputs, 2));
            
            // read CoS register
            coss = AIOWDM.RelInPortDWord(CardNum, 8);
            lblCOS.Text = pad32(Convert.ToString(coss, 2));

            Update();

        }

        private void btnPerformIO_Click(object sender, EventArgs e)
        {
            if (btnPerformIO.Text == "Stop I/O")
            {
                myTimer.Stop();
                btnPerformIO.Text = "Perform I/O";
            }
            else
            {

                myTimer.Enabled = true;
                btnPerformIO.Text = "Stop I/O";
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }

        private void label3_Click(object sender, EventArgs e)
        {
        }

        private void btnClearCoS_Click(object sender, EventArgs e)
        {
            // clear pending CoS notifications by writing a 1 to each bit that is set
            AIOWDM.RelOutPortDWord(CardNum, 8, AIOWDM.RelInPortDWord(CardNum, 8));
        }

        private void tbOutputs_TextChanged(object sender, EventArgs e)
        {
            
        }

        private void tbOutputs_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar == (char)Keys.Delete) || (e.KeyChar == (char)Keys.Back)) 
                return; // always let delete and backspace happen

            if ((sender as TextBox).Text.Length >= 32) e.Handled = true;
            if ((e.KeyChar != (char)Keys.D0) && (e.KeyChar != (char)Keys.D1))
                e.Handled = true;
        }

        private void btnWriteOutputs_Click(object sender, EventArgs e)
        {
            String sdata = tbOutputs.Text;
            UInt32 data = 0;
            int Len, bit = 0;

            Len = tbOutputs.Text.Length;

            for (bit = Len; bit < 32; bit++) 
                sdata = "0" + sdata;
                
            for (bit = 0; bit < 32; bit++)
                if (sdata[bit] == '1') 
                    data += (uint)1 << (31-bit);

            tbOutputs.Text = sdata;
            label2.Text = "0x" + Convert.ToString(data, 16) + " written to card";

            AIOWDM.RelOutPortDWord(CardNum, 0, data);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            AIOWDM.RelOutPortB(CardNum, 0x0c, checkBox1.Checked ? (byte)3 : (byte)1); // "3" is "TTL as outputs, buffers enabled (not tristate)"; "1" is "buffers enabled"
        }
    }
}
